package
{
	import mx.core.*;
	import mx.collections.*;
	import flash.display.*;
	import flash.geom.*
	import flash.text.*;	

	public class BlitRenderer
	{
		private static var _instance:BlitRenderer;
		
		public static var PIXEL_SIZE:int = 2;
		public static var SCR_WIDTH:int = 240;
		public static var SCR_HEIGHT:int = 160;
		
		public static var SPRITE_TEST:int = 0;
				
		[Embed(source="graphics/spritesheet.png")]
		[Bindable]
		public var spriteSheetClass:Class;		
		public var spriteSheetSrc:BitmapAsset;
		public var spriteSheet:BitmapData;
		
		[Embed(source="graphics/AirstreamNF.ttf", fontName="Test", mimeType="application/x-font")]
		private var embedFontClass:Class;
		private var embedFont:Font;		
	
		// double buffer
		public var backBuffer:BitmapData;
		public var postBuffer:BitmapData;
				
		// colour to use to clear backbuffer with
		public var clearColor:uint = 0xffc9057d;
		public var renderColor:uint = 0xff13b506;
		// background
		public var bgSrc:BitmapAsset;
		public var bg:BitmapData;
				
		private var sprites:Array;
		
		private var fadeSpeed:Number;
		private var fade:Number;
		private var fadeCol:uint;		
		
		public function init():void
		{	
			spriteSheetSrc = new spriteSheetClass() as BitmapAsset;
			spriteSheet = spriteSheetSrc.bitmapData;
					
			backBuffer = new BitmapData(SCR_WIDTH, SCR_HEIGHT, false);
			if(PIXEL_SIZE != 1)
				postBuffer = new BitmapData(SCR_WIDTH*PIXEL_SIZE, SCR_HEIGHT*PIXEL_SIZE, false);	
				
			sprites = new Array();
			sprites[SPRITE_TEST] = new SpriteDef(0,0,16,16,1);

			fade = 0;
			fadeSpeed = 0.005;
			fadeCol = 0xff000000;	
		}
		
		public function cls():void
		{
			backBuffer.fillRect(new Rectangle(0,0,SCR_WIDTH,SCR_HEIGHT), clearColor);
		}
				
		public function flip(graphics:Graphics):void
		{
			
			if(PIXEL_SIZE == 1)
			{
				graphics.beginBitmapFill(backBuffer, null, false, false);
				graphics.drawRect(0, 0, SCR_WIDTH, SCR_HEIGHT);
				graphics.endFill();
			}
			else
			{
				var matrix:Matrix = new Matrix();
				matrix.scale (PIXEL_SIZE, PIXEL_SIZE);
				postBuffer.draw(backBuffer, matrix);
						
				graphics.beginBitmapFill(postBuffer, null, false, false);
				graphics.drawRect(0, 0, SCR_WIDTH*PIXEL_SIZE, SCR_HEIGHT*PIXEL_SIZE);
				graphics.endFill();
			}
			
			if(fade > 0)
			{					
				graphics.beginFill(fadeCol, fade);
				graphics.drawRect(0, 0, SCR_WIDTH*PIXEL_SIZE, SCR_HEIGHT*PIXEL_SIZE);
				graphics.endFill()
			} else if(fade < 0)
			{
				graphics.beginFill(fadeCol, 1+fade);
				graphics.drawRect(0, 0, SCR_WIDTH*PIXEL_SIZE, SCR_HEIGHT*PIXEL_SIZE);
				graphics.endFill()					
			}		
		}
		
		public static function getBuffer():BitmapData
		{
			return getInstance().backBuffer;
		}
		
		public function drawSprite(sprite:uint, x:int, y:int, frame:int = 0):void
		{
			var spr:SpriteDef = sprites[sprite];
		    var renderFrame:int = frame % spr.frames;
			backBuffer.copyPixels(spriteSheet, new Rectangle(spr.wdth*renderFrame+spr.xpos,spr.ypos,spr.wdth,spr.hght), new Point(x,y)); 
		}
		
		public function fillRect(rect:Rectangle, fillCol:uint = 0):void
		{
			if(fillCol == 0)
				fillCol = renderColor;
			backBuffer.fillRect(rect, fillCol);
		}
		
		public function drawSpriteText(str:String, x:int, y:int):void
		{
			// If I want to use this I'll have to draw a font!
			/*str = str.toUpperCase();
			var i:int;
			for(i=0; i<str.length; i++)
			{
				var sprite:int = SPRITE_FONT_REGULAR;
				var frame:int = -1;
				var charCode:Number = str.charCodeAt(i);
				
				if(charCode >= 65 && charCode <= 90) // A to Z
					frame = charCode-65;
				if(charCode >= 48 && charCode <= 57) // 0 to 9
					frame = charCode-48+26;
				if(charCode >= 33 && charCode <= 47) // ! to /
				{
					sprite = SPRITE_FONT_SPECIAL;
					frame = charCode-33;
				}
				if(charCode >= 58 && charCode <= 64) // : to @
				{
					sprite = SPRITE_FONT_SPECIAL;
					frame = charCode-58+15;
				}
				if(frame != -1)
					drawSprite(sprite, x, y, frame);
				x += 8;
			}*/
		}
		
		public function drawFontText(str:String, x:int, y:int, center:Boolean = false, col:uint = 0xffffffff, sze:uint=20):void
		{
			var txt:TextField;
			var txtFormat:TextFormat;
			
			txtFormat = new TextFormat();
			txtFormat.size = sze;
			txtFormat.font = "Test";
			txtFormat.bold = true;			
			
			txt = new TextField();
			txt.embedFonts = true;
			txt.autoSize = TextFieldAutoSize.LEFT;

			txt.textColor = col;
			txt.text = str;
			txt.setTextFormat(txtFormat);
			
			if(center)
			{
				x -= txt.textWidth/2;
			}
			
			var matrix:Matrix = new Matrix();
			matrix.translate(x, y);			
			backBuffer.draw(txt, matrix);											
		}
		
		public function startFade(col:uint, speed:Number):void
		{
			if(speed > 0)
				fade = 1;
			else
				fade = -1;
			fadeSpeed = speed;
			fadeCol = col;
		}
		
		public function update():void
		{
			if(fade > 0)
			{
				fade -= fadeSpeed;
				if(fade < 0)
					fade = 0;
			} else if(fade < 0)
			{
				fade -= fadeSpeed;
				if(fade > 0)
					fade = -0.001;
			}
		}		
				
		public static function getInstance():BlitRenderer
		{
			if(! BlitRenderer._instance)
				BlitRenderer._instance = new BlitRenderer();
			return BlitRenderer._instance;			
		}		
	}	
}