package
{
	import mx.core.*;
	import mx.collections.*;
	import flash.geom.*
	
	public class TileMap extends Entity
	{
		private static var _instance:TileMap;
		
		public static var TILE_EMPTY:int=0;
		public static var TILE_FILLED:int=1;
		public static var TILE_MAX:int=2;
		
		public static var COL_NONE:int=0;
		public static var COL_WALL:int=1;
		
		public static var width:int=10;
		public static var height:int=10;
		public static var tileWidth:int=16;
		public static var tileHeight:int=16;
		
		private var tileData:Array;		
		
		private var editTile:int = TILE_FILLED;
		
		public function TileMap()
		{
			// Make an array to store map
			tileData = new Array();
			for(var i:int=0; i<width; i++)
			{
				tileData.push(new Array());
				for(var j:int=0; j<height; j++)
				{
					tileData[i].push(TILE_EMPTY);
				}
			}
			
			tileData[0][3] = TILE_FILLED;
		}
		
		public override function update():void
		{
			if(Input.getInstance().mouseHeld)
			{
				var mousePos:Point = Input.getInstance().mousePos;
				var mouseX:int = mousePos.x / tileWidth;
				var mouseY:int = mousePos.y / tileHeight;
				if(mouseX >= 0 && mouseX < width && mouseY >= 0 && mouseY < height)
					tileData[mouseX][mouseY] = editTile;
			}
			for(var i:int=0; i<TILE_MAX; i++)
			{
				if(Input.getInstance().keyDownDictionary[48+i])
					editTile = i;
			}
		}
		
		public override function render():void
		{
			for(var i:int=0; i<width; i++)
			{
				for(var j:int=0; j<height; j++)
				{
					var sprite:int = -1;
					switch(tileData[i][j])
					{
						default: break;
						case TILE_FILLED: sprite = BlitRenderer.SPRITE_TEST; break;
					}
					if(sprite != -1)
						BlitRenderer.getInstance().drawSprite(sprite, i*tileWidth, j*tileHeight);
				}
			}
		}
		
		public function getTileAtPos(p:Point):int
		{
			var iTileX:int = p.x / tileWidth;
			var iTileY:int = p.y / tileHeight;
			if(iTileX < 0 || iTileX >= width || iTileY < 0 || iTileY >= height)
				return TILE_MAX;
			return tileData[iTileX][iTileY];
		}
		
		public function getColAtPos(p:Point):int
		{
			switch(getTileAtPos(p))
			{
				case TILE_FILLED: return COL_WALL;
			}
			return COL_NONE;
		}
		

		
		public function intToChar(i:uint):String
		{
			if(i<26)
				return String.fromCharCode(65+i); // A-Z
			i -= 26;
			if(i<26)
				return String.fromCharCode(97+i); // a-z
			i -= 26;
			if(i<10)
				return String.fromCharCode(48+i); // 0-9
			i -= 10;
			if(i==0)
				return String.fromCharCode(45); // -
			if(i==1)
				return String.fromCharCode(126); // ~
			trace("intToChar Error, int too high!");
			return null;
		}
		
		public function charToInt(s:String):uint
		{
			var c:uint = s.charCodeAt();
			if(c >= 65 && c < 91)
				return c-65;
			if(c >= 97 && c < 123)
				return c-97+26;
			if(c >= 48 && c < 58)
				return c-48+26+26;
			if(c == 45)
				return 62;
			if(c == 126)
				return 63;				
			trace("charToInt error, invalid char");
			return 0;
		}				
		
				
		public function writeToString():String
		{
			var currentTile:int = -1;
			var numFound:int = 0;
			
			var out:String = "";
			for(var i:int=0; i<width; i++)
			{
				for(var j:int=0; j<height; j++)
				{
					if(tileData[i][j] != currentTile || numFound == 8)
					{
						if(numFound)
						{
							var combined:int = 8*currentTile + (numFound-1);
							out = out.concat(intToChar(combined));
						}
						currentTile = tileData[i][j];
						numFound = 1;
					}
					else
					{
						numFound++;
					}
				}
			}
			out = out.concat(intToChar(8*currentTile + (numFound-1)));
			return out;
		}
		
		public function readFromString(input:String):void
		{
			var i:int=0;
			var j:int=0;
			while(input.length)
			{
				var combined:int = charToInt(input);
				var currentTile:int = combined/8;				
				var numFound:int = combined - currentTile*8 + 1;
				while(numFound)
				{
					tileData[i][j] = currentTile;
					j++;
					if(j==height)
					{
						j=0;
						i++;
						if(i==width)
							return; // too much input just break
					}
					numFound--;
				}
				input = input.slice(1);
			}
		}		
		
		public static function getInstance():TileMap
		{
			if(! TileMap._instance)
				TileMap._instance = new TileMap();
			return TileMap._instance;
		}      		
	}
}